@@ -23,8 +23,9 @@
 #include "game.h"
 #include "map.h"
 
-ColorModifier::ColorModifier(int col, int x, int y, Real min, Real max, Real freq, Real phase)
-    : GameHook(Role_GameHook) {
+ColorModifier::ColorModifier(Game& g, int col, int x, int y, Real min, Real max, Real freq,
+                             Real phase)
+    : GameHook(g, Role_GameHook) {
   this->x = x;
   this->y = y;
   this->min = min;
@@ -42,10 +43,10 @@ ColorModifier::ColorModifier(int col, int x, int y, Real min, Real max, Real fre
 void ColorModifier::tick(Real t) {
   GameHook::tick(t);
 
-  double tt = Game::current->gameTime;
-  Cell& c = Game::current->map->cell(x, y);
+  double tt = game.gameTime;
+  Cell& c = game.map->cell(x, y);
   if (!is_on) return;
-  float fv = min + (max - min) * (1. + cos(phase + (tt * freq) * 2. * M_PI)) / 2.;
+  float fv = min + (max - min) * (1. + std::cos(phase + (tt * freq) * 2. * M_PI)) / 2.;
   uint16_t v = std::max(0.f, std::min(1.f, fv)) * 65535.f;
   switch (colors) {
   case 0:
@@ -65,5 +66,5 @@ void ColorModifier::tick(Real t) {
     for (int i = 0; i < 5; i++) c.colors[i].v[2] = v;
     break;
   }
-  Game::current->map->markCellsUpdated(x, y, x, y, false);
+  game.map->markCellsUpdated(x, y, x, y, false);
 }
