@@ -267,10 +267,10 @@ void EditMode::loadMap(char* name) {
   levelname[255] = '\0';
 
   /* Set the pathname under which we will save the map */
-  snprintf(pathname, sizeof(pathname), "%s/.trackballs/levels/%s.map", getenv("HOME"), name);
+  snprintf(pathname, sizeof(pathname), "%s/levels/%s.map", effectiveLocalDir, name);
 
   // Default load the map from the home directory if existing (same as default pathname)
-  snprintf(mapname, sizeof(mapname), "%s/.trackballs/levels/%s.map", getenv("HOME"), name);
+  snprintf(mapname, sizeof(mapname), "%s/levels/%s.map", effectiveLocalDir, name);
 
   if (!fileExists(mapname))
     // Alternativly from the share directory
@@ -304,26 +304,17 @@ void EditMode::saveMap() {
   char mapname[768];
   char str[768];
 
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs", getenv("HOME"));
+  snprintf(str, sizeof(str) - 1, "%s/levels", effectiveLocalDir);
   if (pathIsLink(str)) {
-    warning("Error, %s/.trackballs is a symbolic link. Cannot save map", getenv("HOME"));
+    warning("Error, %s/levels is a symbolic link. Cannot save map", effectiveLocalDir);
     return;
   } else if (!pathIsDir(str))
     mkdir(str, S_IXUSR | S_IRUSR | S_IWUSR | S_IXGRP | S_IRGRP | S_IWGRP);
 
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs/levels", getenv("HOME"));
+  snprintf(mapname, sizeof(mapname) - 1, "%s/levels/%s.map", effectiveLocalDir, levelname);
   if (pathIsLink(str)) {
-    warning("Error, %s/.trackballs/levels is a symbolic link. Cannot save map",
-            getenv("HOME"));
-    return;
-  } else if (!pathIsDir(str))
-    mkdir(str, S_IXUSR | S_IRUSR | S_IWUSR | S_IXGRP | S_IRGRP | S_IWGRP);
-
-  snprintf(mapname, sizeof(mapname) - 1, "%s/.trackballs/levels/%s.map", getenv("HOME"),
-           levelname);
-  if (pathIsLink(str)) {
-    warning("Error, %s/.trackballs/levels/%s.map is a symbolic link. Cannot save map",
-            getenv("HOME"), levelname);
+    warning("Error, %s/levels/%s.map is a symbolic link. Cannot save map", effectiveLocalDir,
+            levelname);
     return;
   }
 
@@ -334,7 +325,7 @@ void EditMode::saveMap() {
   /* Check if there already exists a script file for this map */
   snprintf(str, sizeof(str), "%s/levels/%s.scm", effectiveShareDir, levelname);
   if (!pathIsFile(str)) {
-    snprintf(str, sizeof(str), "%s/.trackballs/levels/%s.scm", getenv("HOME"), levelname);
+    snprintf(str, sizeof(str), "%s/levels/%s.scm", effectiveLocalDir, levelname);
     if (!pathIsFile(str)) {
       /* No script file exists. Create a default one */
       if (pathIsLink(str)) {
@@ -349,8 +340,7 @@ void EditMode::saveMap() {
               _("Read the documentation for the editor and look at the examples"));
       fprintf(fp, ";;; %s\n\n", _("to learn how to customize it"));
       fprintf(fp, "(set-track-name \"%s\")\n", levelname);
-      fprintf(fp, "(set-author \"%s\")\n      ;; %s\n", getenv("USER"),
-              _("Enter your name here"));
+      fprintf(fp, "(set-author \"%s\")\n      ;; %s\n", username, _("Enter your name here"));
       fprintf(fp, "(start-time 180)\n");
       fprintf(fp, "(set-start-position 250.5 250.5)\n");
       fprintf(fp, "(add-goal 249.0 250.0 #f \"\") ;; %s\n",
@@ -368,9 +358,12 @@ void EditMode::display() {
   } else
     h = 0.0;
 
+  /* Show extra cell flag overlay */
+  activeView.show_flag_state = true;
+
   /* configure lighting */
   activeView.fog_enabled = 0;
-  MainMode::setupLighting(false);
+  MainMode::setupLighting(NULL, false);
 
   /* Setup matrixes for the camera perspective */
   perspectiveMatrix(40, (GLdouble)screenWidth / (GLdouble)std::max(screenHeight, 1), 0.1, 200,
@@ -404,7 +397,8 @@ void EditMode::display() {
 
   /* Draw the map and the current mapcursor/selected region */
   if (map) {
-    map->draw(0, x, y);
+    Coord3d map_focus = {(Real)x, (Real)y, map->cell(x, y).heights[Cell::CENTER]};
+    map->draw(0, map_focus, time);
     map->drawLoop(0, 0, map->width - 1, map->height - 1, 0);
     if (game) {
       /* Indicate start position and trigger object locations */
@@ -425,7 +419,9 @@ void EditMode::display() {
       map->drawSpotRing(map->startPosition[0], map->startPosition[1], 0.5, 0);
       game->draw();
     }
-    map->draw(1, x, y);
+    map->draw(1, map_focus, time);
+    activeView.show_flag_state = false;
+
     /* If we have a clipboard selection then display where it will be pasted */
     if (cellClipboard) {
       map->drawFootprint(x, y, x + cellClipboardWidth - 1, y + cellClipboardHeight - 1, 1);
@@ -1099,7 +1095,7 @@ void EditMode::key(int key) {
 
   return;
 }
-void EditMode::idle(Real td) {
+void EditMode::tick(Real td) {
   int x, y;
   tickMouse(td);
   Uint8 mouseState = SDL_GetMouseState(&x, &y);
@@ -1122,27 +1118,29 @@ void EditMode::makeHill(int radius) {
   for (mx = -radius; mx <= radius; mx++)
     for (my = -radius; my <= radius; my++) {
       map->cell(x + mx, y + my).heights[Cell::CENTER] +=
-          ((int)(sin(1. * M_PI * (mx + radius + 0.5) / diameter) *
-                 sin(1. * M_PI * (my + radius + 0.5) / diameter) * (shift ? -raise : raise) *
-                 10.)) *
+          ((int)(std::sin(1. * M_PI * (mx + radius + 0.5) / diameter) *
+                 std::sin(1. * M_PI * (my + radius + 0.5) / diameter) *
+                 (shift ? -raise : raise) * 10.)) *
           .1;
       map->cell(x + mx, y + my).heights[Cell::NE] +=
-          ((int)(sin(1. * M_PI * (mx + radius + 1.0) / diameter) *
-                 sin(1. * M_PI * (my + radius + 1.0) / diameter) * (shift ? -raise : raise) *
-                 10.)) *
+          ((int)(std::sin(1. * M_PI * (mx + radius + 1.0) / diameter) *
+                 std::sin(1. * M_PI * (my + radius + 1.0) / diameter) *
+                 (shift ? -raise : raise) * 10.)) *
           .1;
       map->cell(x + mx, y + my).heights[Cell::NW] +=
-          ((int)(sin(1. * M_PI * (mx + radius) / diameter) *
-                 sin(1. * M_PI * (my + radius + 1.0) / diameter) * (shift ? -raise : raise) *
-                 10.)) *
+          ((int)(std::sin(1. * M_PI * (mx + radius) / diameter) *
+                 std::sin(1. * M_PI * (my + radius + 1.0) / diameter) *
+                 (shift ? -raise : raise) * 10.)) *
           .1;
       map->cell(x + mx, y + my).heights[Cell::SE] +=
-          ((int)(sin(1. * M_PI * (mx + radius + 1.0) / diameter) *
-                 sin(1. * M_PI * (my + radius) / diameter) * (shift ? -raise : raise) * 10.)) *
+          ((int)(std::sin(1. * M_PI * (mx + radius + 1.0) / diameter) *
+                 std::sin(1. * M_PI * (my + radius) / diameter) * (shift ? -raise : raise) *
+                 10.)) *
           .1;
       map->cell(x + mx, y + my).heights[Cell::SW] +=
-          ((int)(sin(1. * M_PI * (mx + radius) / diameter) *
-                 sin(1. * M_PI * (my + radius) / diameter) * (shift ? -raise : raise) * 10.)) *
+          ((int)(std::sin(1. * M_PI * (mx + radius) / diameter) *
+                 std::sin(1. * M_PI * (my + radius) / diameter) * (shift ? -raise : raise) *
+                 10.)) *
           .1;
     }
   map->markCellsUpdated(x - radius, y - radius, x + radius, y + radius, false);
@@ -1175,13 +1173,14 @@ void EditMode::doSmooth(int radius) {
   for (mx = -radius; mx <= radius; mx++)
     for (my = -radius; my <= radius; my++) {
       Cell& c = map->cell(x + mx, y + my);
-      double s1 = sin(1. * M_PI * (mx + radius + 0.5) / diameter) *
-                  sin(1. * M_PI * (my + radius + 0.5) / diameter) * (shift ? -raise : raise);
+      double s1 = std::sin(1. * M_PI * (mx + radius + 0.5) / diameter) *
+                  std::sin(1. * M_PI * (my + radius + 0.5) / diameter) *
+                  (shift ? -raise : raise);
       c.heights[Cell::CENTER] = steps(c.heights[Cell::CENTER] * (1. - s1) + avgHeight * s1);
       for (north = 0; north < 2; north++)
         for (east = 0; east < 2; east++) {
-          double s = sin(1. * M_PI * (my + radius + 1.0 * north) / diameter) *
-                     sin(1. * M_PI * (mx + radius + 1.0 * east) / diameter) *
+          double s = std::sin(1. * M_PI * (my + radius + 1.0 * north) / diameter) *
+                     std::sin(1. * M_PI * (mx + radius + 1.0 * east) / diameter) *
                      (shift ? -raise : raise);
           c.heights[Cell::NORTH * north + Cell::EAST * east] = steps(
               c.heights[Cell::NORTH * north + Cell::EAST * east] * (1. - s) + avgHeight * s);
