@@ -25,7 +25,7 @@
 
 class Flag : public Animated {
  public:
-  Flag(Real x, Real y, int points, int visible, Real radius);
+  Flag(Game& g, Real x, Real y, int points, int visible, Real radius);
 
   virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
   virtual void drawBuffers1(const GLuint*) const;
