@@ -25,7 +25,8 @@
 
 class Fountain : public Animated {
  public:
-  Fountain(const Coord3d& position, double randomSpeed, double radius, double strength);
+  Fountain(Game& g, const Coord3d& position, double randomSpeed, double radius,
+           double strength);
 
   virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
   virtual void drawBuffers1(const GLuint*) const;
