@@ -31,6 +31,7 @@
 #include "map.h"
 #include "pipe.h"
 #include "player.h"
+#include "replay.h"
 #include "settings.h"
 #include "sound.h"
 #include "weather.h"
@@ -41,17 +42,15 @@ extern Color colors[5];
 
 double Game::defaultScores[SCORE_MAX][2];
 
-Game::Game(const char *name, Gamer *g) {
+Game::Game(const char *name, Gamer *gamer) {
   balls = new AnimatedCollection();
 
-  current = this;
-
   map = NULL;
   player1 = NULL;
   gameTime = 0.0;
+  gameTicks = 0;
   nextLevel[0] = 0;
   setDefaults();
-  gamer = g;
 
   localPlayers = 1;
   edit_mode = 0;
@@ -63,13 +62,12 @@ Game::Game(const char *name, Gamer *g) {
   snprintf(scmname, sizeof(scmname), "%s/levels/boot.scm", effectiveShareDir);
   loadScript(scmname);
 
-  player1 = new Player();
+  player1 = new Player(*this);
   add(player1);
 
-  loadLevel(name);
-  player1->restart(Game::current->map->startPosition);
+  loadLevel(name, gamer);
 
-  player1->restart(Game::current->map->startPosition);
+  player1->restart(map->startPosition);
   player1->timeLeft = startTime;
   player1->lives = 4 - Settings::settings->difficulty;
 }
@@ -79,20 +77,19 @@ Game::Game(Map *editmap, const char *levelname) {
   localPlayers = 0;
   player1 = NULL;
   gameTime = 0.0;
+  gameTicks = 0;
   nextLevel[0] = 0;
   weather = NULL;
   map = editmap;
   edit_mode = 1;
   setDefaults();
 
-  current = this;
-
   /* load scripts */
   char scmname[512];
   snprintf(scmname, sizeof(scmname), "%s/levels/boot.scm", effectiveShareDir);
   scmname[511] = '\0';
   loadScript(scmname);
-  snprintf(scmname, 511, "%s/.trackballs/levels/%s.scm", getenv("HOME"), levelname);
+  snprintf(scmname, 511, "%s/levels/%s.scm", effectiveLocalDir, levelname);
   scmname[511] = '\0';
   if (!fileExists(scmname)) {
     snprintf(scmname, 511, "%s/levels/%s.scm", effectiveShareDir, levelname);
@@ -119,14 +116,13 @@ Game::~Game() {
   }
 
   if (!edit_mode) delete map;
-  if (current == this) { current = NULL; }
 }
 
-void Game::loadLevel(const char *name) {
+void Game::loadLevel(const char *name, Gamer *gamer) {
   char mapname[256];
   char scmname[256];
 
-  if (player1 && gamer) {
+  if (gamer) {
     /* level scripts might have modified our appearance. Reset them */
     player1->primaryColor = colors[gamer->color];
     player1->texture = gamer->textureNum;
@@ -143,17 +139,37 @@ void Game::loadLevel(const char *name) {
    * the home directory first (eg. ~/.trackballs/levels/{name}.map)
    * It is possible to mix using a map file in the home dir and a
    * script in the share dir or viceversa */
-  snprintf(mapname, sizeof(mapname) - 1, "%s/.trackballs/levels/%s.map", getenv("HOME"), name);
-  snprintf(scmname, sizeof(scmname) - 1, "%s/.trackballs/levels/%s.scm", getenv("HOME"), name);
+  snprintf(mapname, sizeof(mapname) - 1, "%s/levels/%s.map", effectiveLocalDir, name);
+  snprintf(scmname, sizeof(scmname) - 1, "%s/levels/%s.scm", effectiveLocalDir, name);
   if (!fileExists(mapname))
     snprintf(mapname, sizeof(mapname), "%s/levels/%s.map", effectiveShareDir, name);
   if (!fileExists(scmname))
     snprintf(scmname, sizeof(scmname), "%s/levels/%s.scm", effectiveShareDir, name);
   snprintf(levelName, sizeof(levelName), "%s", name);
 
+  if (Settings::settings->storeReplay) {
+    if (Settings::settings->replay) {
+      warning("Reading replay");
+      replayStore.read(levelName);
+      Settings::settings->sandbox = replayStore.getSettings().sandbox;
+      Settings::settings->difficulty = replayStore.getSettings().difficulty;
+      // We do not need to store the existing settings, because they will be
+      // set by the next time the corresponding menu item changes
+    } else {
+      warning("Preparing to store replay");
+      struct GameSettings gs;
+      gs.sandbox = Settings::settings->sandbox;
+      gs.difficulty = Settings::settings->difficulty;
+      replayStore.init(gs);
+    }
+  }
+
   if (map) delete map;
   map = new Map(mapname);
+
+  Game::current = this;
   loadScript(scmname);
+  Game::current = NULL;
 
   for (int j = 0; j < newHooks.size(); j++) {
     hooks[newHooks[j]->entity_role].push_back(newHooks[j]);
@@ -163,6 +179,8 @@ void Game::loadLevel(const char *name) {
   if (player1) player1->timeLeft = startTime;
 
   fogThickness = wantedFogThickness;
+
+  randSeed = 0;
 }
 
 void Game::setDefaults() {
@@ -244,87 +262,95 @@ void Game::clearLevel() {
     for (int k = 0; k < n; k++) { delete hooks[i][k]; }
     hooks[i].clear();
   }
+
+  if (Settings::settings->storeReplay && !Settings::settings->replay)
+    replayStore.save(levelName);
+}
+
+void Game::handleUserInput(bool active) {
+  if (player1) { player1->handleUserInput(active); }
+}
+void Game::handleKey(int key) {
+  if (player1) { player1->handleKey(key); }
 }
 
 void Game::tick(Real t) {
-  int steps = std::max(1, int(t / PHYSICS_RESOLUTION + 0.5));
-  double origtime = gameTime;
-  double tstep = t / steps;
+  // todo: decouple gameTime, replace with global 'displayTime'
+  gameTime += t;
 
-  /* The game ticks run at a faster time scale so that the interaction
-   * of different moving objects is realistic */
-  for (int i = 0; i < steps; i++) {
-    gameTime += tstep;
+  gameTicks += 1;
 
-    /* Update intersection information */
-    balls->clear();
-    balls->reserve(hooks[Role_Ball].size() + hooks[Role_Player].size());
-    for (int j = Role_Ball; j <= Role_Player; j++) {
-      int nb = hooks[j].size();
-      for (int k = 0; k < nb; k++) { balls->add((Animated *)hooks[j][k]); }
+  /* Update intersection information */
+  balls->clear();
+  balls->reserve(hooks[Role_Ball].size() + hooks[Role_Player].size());
+  for (int j = Role_Ball; j <= Role_Player; j++) {
+    int nb = hooks[j].size();
+    for (int k = 0; k < nb; k++) { balls->add((Animated *)hooks[j][k]); }
+  }
+  balls->recalculateBboxMap();
+
+  /* update active entities */
+  for (int j = Role_GameHook; j < Role_MaxTypes; j++) {
+    int n = hooks[j].size();
+    for (int k = 0; k < n; k++) { hooks[j][k]->tick(t); }
+  }
+
+  /* add new entities */
+  for (int j = 0; j < newHooks.size(); j++) {
+    hooks[newHooks[j]->entity_role].push_back(newHooks[j]);
+  }
+  newHooks.clear();
+
+  /* run queued callbacks */
+  Game::current = this;
+  for (int j = 0; j < queuedCalls.size(); j++) {
+    QueuedCall call = queuedCalls[j];
+    /* the functions are owned by GameHooks; arguments by Game */
+    if (call.type == 0) {
+      scm_catch_call_0(call.fun);
+    } else if (call.type == 1) {
+      scm_catch_call_1(call.fun, scm_from_double(call.val));
+    } else if (call.type == 2) {
+      SCM sub;
+      Animated *a = dynamic_cast<Animated *>(call.subject);
+      if (a)
+        sub = smobAnimated_make(a);
+      else
+        sub = smobGameHook_make(call.subject);
+
+      scm_catch_call_2(call.fun, sub, call.object ? call.object : SCM_BOOL_F);
+      if (call.object) { scm_gc_unprotect_object(call.object); }
     }
-    balls->recalculateBboxMap();
+  }
+  Game::current = NULL;
+  queuedCalls.clear();
 
-    /* update active entities */
-    for (int j = Role_GameHook; j < Role_MaxTypes; j++) {
-      int n = hooks[j].size();
-      for (int k = 0; k < n; k++) { hooks[j][k]->tick(tstep); }
-    }
+  /* filter out dead entities, except players */
+  for (int j = Role_GameHook; j < Role_MaxTypes; j++) {
+    if (j == Role_Player) continue;
 
-    /* add new entities */
-    for (int j = 0; j < newHooks.size(); j++) {
-      hooks[newHooks[j]->entity_role].push_back(newHooks[j]);
-    }
-    newHooks.clear();
-
-    /* run queued callbacks */
-    for (int j = 0; j < queuedCalls.size(); j++) {
-      QueuedCall call = queuedCalls[j];
-      /* the functions are owned by GameHooks; arguments by Game */
-      if (call.type == 0) {
-        scm_catch_call_0(call.fun);
-      } else if (call.type == 1) {
-        scm_catch_call_1(call.fun, scm_from_double(call.val));
-      } else if (call.type == 2) {
-        SCM sub;
-        Animated *a = dynamic_cast<Animated *>(call.subject);
-        if (a)
-          sub = smobAnimated_make(a);
-        else
-          sub = smobGameHook_make(call.subject);
-
-        scm_catch_call_2(call.fun, sub, call.object ? call.object : SCM_BOOL_F);
-        if (call.object) { scm_gc_unprotect_object(call.object); }
-      }
-    }
-    queuedCalls.clear();
-
-    /* filter out dead entities, except players */
-    for (int j = Role_GameHook; j < Role_MaxTypes; j++) {
-      if (j == Role_Player) continue;
-
-      int n = hooks[j].size();
-      int ndead = 0;
-      for (int k = 0; k < n; k++) {
-        if (hooks[j][k - ndead]->invalid) {
-          GameHook *hook = hooks[j][k - ndead];
-          hook->releaseCallbacks();
-          hooks[Role_Dead].push_back(hook);
-          hooks[j][k - ndead] = hooks[j][n - 1 - ndead];
-          hooks[j].pop_back();
-          ndead++;
-        }
+    int n = hooks[j].size();
+    int ndead = 0;
+    for (int k = 0; k < n; k++) {
+      if (hooks[j][k - ndead]->invalid) {
+        GameHook *hook = hooks[j][k - ndead];
+        hook->releaseCallbacks();
+        hooks[Role_Dead].push_back(hook);
+        hooks[j][k - ndead] = hooks[j][n - 1 - ndead];
+        hooks[j].pop_back();
+        ndead++;
       }
     }
   }
-  gameTime = origtime + t;
 
   /* Weather updates have no physical effect */
   if (fogThickness < wantedFogThickness)
     fogThickness += std::min(0.3 * t, wantedFogThickness - fogThickness);
   if (fogThickness > wantedFogThickness)
     fogThickness -= std::min(0.3 * t, fogThickness - wantedFogThickness);
-  weather->tick(t);
+  // todo: move into a tick based on display timesteps, so draw() can
+  // remain pseudo-const
+  weather->tick(t, player1);
 }
 void Game::doExpensiveComputations() {
   for (int i = 0; i < Role_MaxTypes; i++) {
@@ -373,7 +399,7 @@ void Game::draw() {
       if (anim->onScreen) anim->draw2();
     }
   }
-  if (weather) weather->draw2();
+  if (weather) weather->draw2(player1);
 
   warnForGLerrors("Game drawing");
 }
@@ -426,3 +452,8 @@ void Game::drawReflection(const Coord3d &focus) {
     }
   }
 }
+
+double Game::frandom() {
+  int v = rand_r(&randSeed) % (1 << 30);
+  return v / (double)(1 << 30);
+}
