@@ -32,18 +32,13 @@
 #include <zlib.h>
 #include <cstdlib>
 
+Gamer *Gamer::gamer = NULL;
+
 Gamer::Gamer() {
   memset(name, 0, sizeof(name));
   strncpy(name, _("John Doe"), sizeof(name));
   name[19] = '\0';
-  if (NULL != getenv("USER")) {
-    snprintf(name, 20, "%s", getenv("USER"));
-  }
-#ifdef WIN32
-  else {
-    if (NULL != getenv("USERNAME")) { snprintf(name, 20, "%s", getenv("USERNAME")); }
-  }
-#endif
+  if (username[0]) { snprintf(name, sizeof(name) - 1, "%s", username); }
 
   memset(levels, 0, sizeof(levels));
   for (int i = 0; i < Settings::settings->nLevelSets; i++) levels[i] = new KnownLevel[256];
@@ -68,29 +63,30 @@ void Gamer::setDefaults() {
   nLevelsCompleted = 0;
   for (int i = 0; i < Settings::settings->nLevelSets; i++) {
     nKnownLevels[i] = 1;
-    strcpy(levels[i][0].name, Settings::settings->levelSets[i].startLevelName);
-    strcpy(levels[i][0].fileName, Settings::settings->levelSets[i].startLevel);
+    strncpy(levels[i][0].name, Settings::settings->levelSets[i].startLevelName,
+            sizeof(levels[i][0].name));
+    strncpy(levels[i][0].fileName, Settings::settings->levelSets[i].startLevel,
+            sizeof(levels[i][0].fileName));
   }
   textureNum = 0;
 }
-void Gamer::levelStarted() {
+void Gamer::levelStarted(Game *game) {
   int i;
-  char *level = Game::current->levelName;
-  if (Game::current->currentLevelSet < 0) return;  // don't modify profile when cheating
-  if (Game::current->map->isBonus) return;  // bonus levels are not added to known levels
-  for (i = 0; i < nKnownLevels[Game::current->currentLevelSet]; i++)
-    if (strcmp(levels[Game::current->currentLevelSet][i].fileName, level) == 0) break;
-  if (i == nKnownLevels[Game::current->currentLevelSet]) {
-    strncpy(levels[Game::current->currentLevelSet][i].fileName, level, 64);
-    strncpy(levels[Game::current->currentLevelSet][i].name, Game::current->map->mapname, 64);
-    nKnownLevels[Game::current->currentLevelSet]++;
+  char *level = game->levelName;
+  if (game->currentLevelSet < 0) return;  // don't modify profile when cheating
+  if (game->map->isBonus) return;         // bonus levels are not added to known levels
+  for (i = 0; i < nKnownLevels[game->currentLevelSet]; i++)
+    if (strcmp(levels[game->currentLevelSet][i].fileName, level) == 0) break;
+  if (i == nKnownLevels[game->currentLevelSet]) {
+    strncpy(levels[game->currentLevelSet][i].fileName, level, 64);
+    strncpy(levels[game->currentLevelSet][i].name, game->map->mapname, 64);
+    nKnownLevels[game->currentLevelSet]++;
     nLevelsCompleted++;
     save();
-  } else if (strncmp(levels[Game::current->currentLevelSet][i].name,
-                     Game::current->map->mapname, 64) != 0) {
+  } else if (strncmp(levels[game->currentLevelSet][i].name, game->map->mapname, 64) != 0) {
     /* level name change, i.e. due to translation or order change. */
-    strncpy(levels[Game::current->currentLevelSet][i].fileName, level, 64);
-    strncpy(levels[Game::current->currentLevelSet][i].name, Game::current->map->mapname, 64);
+    strncpy(levels[game->currentLevelSet][i].fileName, level, 64);
+    strncpy(levels[game->currentLevelSet][i].name, game->map->mapname, 64);
     save();
   }
 }
@@ -101,17 +97,10 @@ void Gamer::save() {
   char str[256];
 
   Settings *settings = Settings::settings;
-
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs", getenv("HOME"));
+  snprintf(str, sizeof(str) - 1, "%s/%s.gmr", effectiveLocalDir, name);
   if (pathIsLink(str)) {
-    warning("Error, %s/.trackballs is a symbolic link. Cannot save settings", getenv("HOME"));
-    return;
-  }
-  if (!pathIsDir(str)) mkdir(str, S_IXUSR | S_IRUSR | S_IWUSR | S_IXGRP | S_IRGRP | S_IWGRP);
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs/%s.gmr", getenv("HOME"), name);
-  if (pathIsLink(str)) {
-    warning("Error, %s/.trackballs/%s.gmr is a symbolic link. Cannot save settings",
-            getenv("HOME"), name);
+    warning("Error, %s/%s.gmr is a symbolic link. Cannot save settings", effectiveLocalDir,
+            name);
     return;
   }
 
@@ -145,7 +134,7 @@ void Gamer::save() {
 void Gamer::update() {
   char str[256];
 
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs/%s.gmr", getenv("HOME"), name);
+  snprintf(str, sizeof(str) - 1, "%s/%s.gmr", effectiveLocalDir, name);
   SCM ip = scm_port_from_gzip(str, 256 * 256 * 128);
   if (SCM_EOF_OBJECT_P(ip)) {
     setDefaults();
@@ -232,17 +221,14 @@ void Gamer::update() {
   scm_close_input_port(ip);
   return;
 }
-void Gamer::playerLose() {
+void Gamer::playerLose(Game *game) {
   timesPlayed++;
-  totalScore += Game::current->player1->score;
+  totalScore += game->player1->score;
   save();
 }
 void Gamer::reloadNames() {
-  char str[256];
-
   nNames = 0;
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs", getenv("HOME"));
-  DIR *dir = opendir(str);
+  DIR *dir = opendir(effectiveLocalDir);
   if (dir) {
     struct dirent *dirent;
     while ((dirent = readdir(dir))) {
