@@ -35,15 +35,12 @@ class Game;
 /* Prototypes */
 void glHelpInit();
 void glHelpCleanup();
-GLuint loadProgram(const char *fragment, const char *vertex);
 void warnForGLerrors(const char *where_am_i);
-double frand();
-double frand(size_t);
 
 /* displays a 2d text on specific screen coordinates, returning width */
 int draw2DString(TTF_Font *, const char *, int x, int y, float red, float green, float blue,
                  float alpha, bool outlined, int align, int maxwidth = 0);
-void update2DStringCache();
+void update2DStringCache(bool force_wipe);
 /* Common interface for drawing 2d things on the screen */
 void draw2DRectangle(GLfloat x, GLfloat y, GLfloat w, GLfloat h, GLfloat tx, GLfloat ty,
                      GLfloat tw, GLfloat th, GLfloat r, GLfloat g, GLfloat b, GLfloat a,
@@ -57,6 +54,7 @@ void drawMouse(int x, int y, int w, int h);
 size_t packObjectVertex(void *dest, GLfloat x, GLfloat y, GLfloat z, GLfloat tx, GLfloat ty,
                         const Color &color, const GLfloat normal[3]);
 void configureObjectAttributes();
+void configureUIAttributes();
 inline uint32_t packNormal(const GLfloat n[3]) {
   uint32_t d = (512 + n[2] * 511.f);
   uint32_t e = (512 + n[1] * 511.f);
@@ -90,10 +88,10 @@ typedef enum {
   Lighting_NoShadows, /* Orientation dependent colors, no shadows */
   Lighting_Regular    /* Shadows and orientation dependent colors */
 } Object_Lighting;
-void setObjectUniforms(Color specular, float shininess, Object_Lighting lighting);
+void setObjectUniforms(Color specular, float sharpness, Object_Lighting lighting);
 /* if the currently active shader has a uniform with matching name, then this struct provides
  * the location */
-typedef struct {
+struct UniformLocations {
   GLuint line_color;
   GLuint refl_color;
   GLuint tex;
@@ -102,10 +100,14 @@ typedef struct {
   GLuint arrtex;
   GLuint wtex;
   GLuint specular_color;
-  GLuint shininess;
+  GLuint sharpness;
   GLuint ignore_shadow;
   GLuint use_lighting;
-} UniformLocations;
+  // For UI shader
+  GLuint ui_screen_width = 0;
+  GLuint ui_screen_height = 0;
+  GLuint ui_tex = 0;
+};
 extern UniformLocations uniformLocations;
 
 typedef struct _viewpa {
@@ -133,6 +135,8 @@ typedef struct _viewpa {
   GLuint shadowMapTexture;
   GLuint shadowMapTexsize;
   bool calculating_shadows;
+
+  bool show_flag_state;
 } ViewParameters;
 
 extern ViewParameters activeView;
