@@ -24,16 +24,19 @@
 #include "map.h"
 #include "player.h"
 
-Goal::Goal(Real x, Real y, int rotate, char *nextLevel) : Flag(x, y, 1000, 1, 0.2) {
+#include <cstring>
+
+Goal::Goal(Game &g, Real x, Real y, int rotate, char *nextLevel)
+    : Flag(g, x, y, 1000, 1, 0.2) {
   strncpy(this->nextLevel, nextLevel, sizeof(this->nextLevel));
   this->rotate = rotate;
   primaryColor = Color(0.9, 0.8, 0.3, 1.0);
   specularColor = Color(0.95, 0.9, 0.65, 1.0);
 }
 void Goal::onGet() {
-  if (!Game::current->player1->hasWon) {
-    strncpy(Game::current->nextLevel, nextLevel, sizeof(Game::current->nextLevel));
-    if (Game::current->map->isBonus)
+  if (!game.player1->hasWon) {
+    strncpy(game.nextLevel, nextLevel, sizeof(game.nextLevel));
+    if (game.map->isBonus)
       MainMode::mainMode->bonusLevelComplete();
     else
       MainMode::mainMode->levelComplete();
@@ -44,7 +47,7 @@ void Goal::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
                            const GLuint *vaolist, bool mustUpdate) const {
   if (!mustUpdate || !visible) return;
 
-  const int nfacets = 11;
+  const int nfacets = 22;
   GLfloat inner_arc[2 + nfacets][2];
   GLfloat outer_arc[2 + nfacets][2];
   GLfloat normals[2 + nfacets][2];
@@ -135,7 +138,7 @@ void Goal::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
 void Goal::drawBuffers1(const GLuint *vaolist) const {
   if (!visible) return;
 
-  const int nfacets = 11;
+  const int nfacets = 22;
 
   glEnable(GL_CULL_FACE);
   glDisable(GL_BLEND);
