@@ -117,8 +117,8 @@ void HelpMode::display() {
   }
 
   if (helpGame->fogThickness)
-    glClearColor(Game::current->fogColor[0], Game::current->fogColor[1],
-                 Game::current->fogColor[2], Game::current->fogColor[3]);
+    glClearColor(helpGame->fogColor[0], helpGame->fogColor[1], helpGame->fogColor[2],
+                 helpGame->fogColor[3]);
   else
     glClearColor(0.0, 0.0, 0.0, 0.0);
 
@@ -129,7 +129,7 @@ void HelpMode::display() {
     activeView.fog_end = 26.0 - 4.0 * helpGame->fogThickness;
   } else
     activeView.fog_enabled = 0;
-  MainMode::setupLighting(false);
+  MainMode::setupLighting(NULL, false);
 
   /* Render two views of the level map */
   perspectiveMatrix(25, (GLdouble)screenWidth / (GLdouble)std::max(screenHeight, 1), 0.1, 200,
@@ -153,9 +153,10 @@ void HelpMode::display() {
 
   glViewport(11 * screenWidth / 20, 11 * screenHeight / 20, 4 * screenWidth / 10,
              4 * screenHeight / 10);
-  helpGame->map->draw(0, step / 2 + step * page, step / 2);
+  Coord3d focus_a = {step / 2 + step * page, step / 2, 2};
+  helpGame->map->draw(0, focus_a, helpGame->gameTime);
   helpGame->draw();
-  helpGame->map->draw(1, step / 2 + step * page, step / 2);
+  helpGame->map->draw(1, focus_a, helpGame->gameTime);
 
   lookAtMatrix(cameraFrom2[0], cameraFrom2[1], cameraFrom2[2], cameraTo2[0], cameraTo2[1],
                cameraTo2[2], 0.0, 0.0, 1.0, activeView.modelview);
@@ -169,9 +170,10 @@ void HelpMode::display() {
   renderDummyShadowMap();
   glViewport(11 * screenWidth / 20, 1 * screenHeight / 10, 4 * screenWidth / 10,
              4 * screenHeight / 10);
-  helpGame->map->draw(0, step / 2 + step * page, 3 * step / 2);
+  Coord3d focus_b = {step / 2 + step * page, 3 * step / 2, 2};
+  helpGame->map->draw(0, focus_b, helpGame->gameTime);
   helpGame->draw();
-  helpGame->map->draw(1, step / 2 + step * page, 3 * step / 2);
+  helpGame->map->draw(1, focus_b, helpGame->gameTime);
 
   glViewport(0, 0, screenWidth, screenHeight);
   Enter2DMode();
@@ -262,17 +264,15 @@ void HelpMode::display() {
   int fontSize = computeLineSize();
   if (s1 >= 0) {
     for (int i = s1; text[i]; i++) {
-      Font::drawSimpleText(text[i], 50,
-                           1 * screenHeight / 10 + (i - s1) * 3 * fontSize + fontSize,
-                           fontSize, 0.9, 0.9, 0.25, 1., screenWidth / 2 - 50);
+      drawSimpleText(text[i], 50, 1 * screenHeight / 10 + (i - s1) * 3 * fontSize + fontSize,
+                     fontSize, 0.9, 0.9, 0.25, 1., screenWidth / 2 - 50);
     }
   }
 
   if (s2 >= 0) {
     for (int i = s2; text[i]; i++) {
-      Font::drawSimpleText(text[i], 50,
-                           6 * screenHeight / 10 + (i - s2) * 3 * fontSize + fontSize,
-                           fontSize, 0.9, 0.9, 0.25, 1., screenWidth / 2 - 50);
+      drawSimpleText(text[i], 50, 6 * screenHeight / 10 + (i - s2) * 3 * fontSize + fontSize,
+                     fontSize, 0.9, 0.9, 0.25, 1., screenWidth / 2 - 50);
     }
   }
 
@@ -294,14 +294,14 @@ void HelpMode::key(int key) {
     mouseDown(shift ? 3 : 1, -1, -1);
   if (key == SDLK_ESCAPE) GameMode::activate(MenuMode::init());
 }
-void HelpMode::idle(Real td) {
+void HelpMode::tick(Real td) {
   helpGame->tick(td);
   tickMouse(td);
 
   if (isExiting)
     timeLeft -= td;
   else
-    timeLeft = fmin(1.0, timeLeft + td);
+    timeLeft = std::min(1.0, timeLeft + td);
   if (timeLeft < 0.0) GameMode::activate(MenuMode::init());
 }
 void HelpMode::mouseDown(int button, int /*x*/, int /*y*/) {
