@@ -19,12 +19,10 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include "ball.h"
 #include "calibrateJoystickMode.h"
 #include "editMode.h"
 #include "enterHighScoreMode.h"
 #include "font.h"
-#include "forcefield.h"
 #include "game.h"
 #include "gameMode.h"
 #include "general.h"
@@ -36,8 +34,6 @@
 #include "mainMode.h"
 #include "map.h"
 #include "menuMode.h"
-#include "pipe.h"
-#include "pipeConnector.h"
 #include "settings.h"
 #include "settingsMode.h"
 #include "setupMode.h"
@@ -51,6 +47,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <queue>
 
 #ifdef WIN32
 #define WIN32_LEAN_AND_MEAN 1
@@ -59,17 +56,18 @@
 #endif
 
 /* Important globals */
-SDL_Window *window = NULL;
-SDL_Surface *screen = NULL;
-SDL_GLContext mainContext;
-const char *program_name;
+static SDL_Window *window = NULL;
 int debug_joystick, repair_joystick;
-int not_yet_windowed = 1;
+static int not_yet_windowed = 1;
 struct timespec displayStartTime, lastDisplayStartTime;
+static bool has_audio = true;
+static bool startInEditMode;
 
-int theFrameNumber = 0;
+int displayFrameNumber = 0;
 
 char effectiveShareDir[256];
+char effectiveLocalDir[256];
+char username[256];
 int screenResolutions[5][2] = {{640, 480},
                                {800, 600},
                                {1024, 768},
@@ -117,26 +115,18 @@ void changeScreenResolution() {
     }
   }
 
-  screen = SDL_GetWindowSurface(window);
-
-  if (!screen) return;
-  /* Pick up what the screen actually has */
-  screenWidth = screen->w;
-  screenHeight = screen->h;
-
   /* Use CapsLock key to determine if mouse should be hidden */
   if (SDL_GetModState() & KMOD_CAPS) {
     SDL_ShowCursor(SDL_ENABLE);
   } else {
     SDL_ShowCursor(SDL_DISABLE);
   }
-  SDL_GL_SetSwapInterval(Settings::settings->vsynced ? 1 : 0);
 
-  /* Adjust for size change in editmode */
-  if (EditMode::editMode) { EditMode::editMode->resizeWindows(); }
+  /* Depending on platform, this might work, or not, or fail silently */
+  SDL_GL_SetSwapInterval(Settings::settings->vsynced ? 1 : 0);
 }
 
-static void createWindow() {
+static SDL_GLContext createWindow() {
   int full = Settings::settings->is_windowed == 0;
   int fixed = Settings::settings->resolution >= 0;
 
@@ -196,10 +186,18 @@ static void createWindow() {
 
   if (window == NULL) {
     warning("Could not create window: %s", SDL_GetError());
-    return;
+    return NULL;
   }
 
-  mainContext = SDL_GL_CreateContext(window);
+  /* Because windowing functions are only reliable on the main thread (e.g. X11,
+   * without special thread init code), the OpenGL context is created here. In
+   * order to use the OpenGL context on the off-thread, we call
+   * `SDL_GL_MakeCurrent` there; because GLES (for Wayland) contexts are limited
+   * to a single thread, we clear the current (thread) context; see also
+   * the SDL source code. */
+  SDL_GLContext ctx = SDL_GL_CreateContext(window);
+  if (!ctx) { error("Failed to create OpenGL context: %s\n", SDL_GetError()); }
+  SDL_GL_MakeCurrent(window, NULL);
 
   char str[256];
   snprintf(str, sizeof(str), "%s/icons/trackballs-128x128.png", effectiveShareDir);
@@ -210,9 +208,12 @@ static void createWindow() {
   }
 
   changeScreenResolution();
+  SDL_GetWindowSize(window, &screenWidth, &screenHeight);
+
+  return ctx;
 }
 
-static void print_usage(FILE *stream) {
+static void print_usage(FILE *stream, const char *program_name) {
   fprintf(stream, "%s %s %s\n", _("Usage:"), program_name,
           _("[-w, -m] [-e, -l -t <level>] [-r <width>] [-s <sensitivity>]"));
   const char *options[12][2] = {
@@ -247,7 +248,7 @@ static void print_usage(FILE *stream) {
   }
 }
 
-int testDir() {
+static int testDir() {
   if (!strlen(effectiveShareDir)) return 0;
 #ifdef WIN32
   // Really this is just done for asthetics
@@ -271,129 +272,16 @@ int testDir() {
   return 1;
 }
 
-void innerMain(void * /*closure*/, int argc, char **argv) {
-  int editMode = 0, touchMode = 0;
-  int audio = SDL_INIT_AUDIO;
-  SDL_Event event;
-  char *touchName = 0;
+static void *mainLoop(void* data) {
+  /* OpenGL work is now *only* performed on this thread. */
+  SDL_GLContext context = (SDL_GLContext)data;
+  int r = SDL_GL_MakeCurrent(window, context);
+  if (r < 0) { error("Failed to make OpenGL context current: %s", SDL_GetError()); }
+  SDL_GL_SetSwapInterval(Settings::settings->vsynced ? 1 : 0);
 
-  const char *const short_options = "he:l:t:wmr:s:fqvyj";
-  const struct option long_options[] = {{"help", 0, NULL, 'h'},
-                                        {"level", 1, NULL, 'l'},
-                                        {"windowed", 0, NULL, 'w'},
-                                        {"mute", 0, NULL, 'm'},
-                                        {"resolution", 1, NULL, 'r'},
-                                        {"sensitivity", 1, NULL, 's'},
-                                        {"fps", 0, NULL, 'f'},
-                                        {"version", 0, NULL, 'v'},
-                                        {"touch", 1, NULL, 't'},
-                                        {"low-memory", 0, NULL, 'y'},
-                                        {"debug-joystick", 0, NULL, '9'},
-                                        {"repair-joystick", 0, NULL, 'j'},
-                                        {NULL, 0, NULL, 0}};
-  int next_option;
-
-  program_name = argv[0];
-
-  displayStartTime = getMonotonicTime();
-  lastDisplayStartTime = displayStartTime;
-  lastDisplayStartTime.tv_sec -= 1;
-  Settings::init();
-  Settings *settings = Settings::settings;
-  settings->doSpecialLevel = 0;
-  settings->setLocale(); /* Start "correct" i18n as soon as possible */
-  low_memory = 0;
-  debug_joystick = 0;
-  repair_joystick = 0;
-  do {
-#if defined(__SVR4) && defined(__sun)
-    next_option = getopt(argc, argv, short_options);
-#else
-    next_option = getopt_long(argc, argv, short_options, long_options, NULL);
-#endif
-
-    int i;
-    switch (next_option) {
-    case 'h':
-      print_usage(stdout);
-      exit(0);
-      break;
-    case 'l':
-      snprintf(Settings::settings->specialLevel, sizeof(Settings::settings->specialLevel) - 1,
-               "%s", optarg);
-      Settings::settings->doSpecialLevel = 1;
-      break;
-    case 't':
-      touchMode = 1;
-      touchName = optarg;
-      audio = 0;  // no audio
-      break;
-    case 'w':
-      settings->is_windowed = 1;
-      break;
-    case 'm':
-      audio = 0;
-      break;
-    case 'r':
-      for (i = 0; i < nScreenResolutions; i++)
-        if (screenResolutions[i][0] == atoi(optarg)) break;
-      if (i < nScreenResolutions)
-        settings->resolution = i;
-      else {
-        char estr[256];
-        snprintf(estr, 255, _("Unknown screen resolution of width %d"), i);
-        printf("%s\n", estr);
-      }
-      break;
-    case 's':
-      Settings::settings->mouseSensitivity = atof(optarg);
-      break;
-    case 'f':
-      Settings::settings->showFPS = 1;
-      break;
-    case '?':
-      print_usage(stderr);
-      exit(1);
-    case -1:
-      break;
-    case 'v':
-      printf("%s v%s\n", PACKAGE, VERSION);
-      exit(0);
-      break;
-    case 'y':
-      low_memory = 1;
-      break;
-    case '9':
-      debug_joystick = 1;
-      break;
-    case 'j':
-      repair_joystick = 1;
-      break;
-    default:
-      print_usage(stderr);
-      exit(1);
-    }
-  } while (next_option != -1);
-
-  printf("%s\n", _("Welcome to Trackballs."));
-  char str[256];
-  snprintf(str, 255, _("Using %s as gamedata directory."), effectiveShareDir);
-  printf("%s\n", str);
-
-  /* Initialize SDL */
-  if ((SDL_Init(SDL_INIT_VIDEO | audio | SDL_INIT_JOYSTICK) == -1)) {
-    error("Could not initialize libSDL. Error message: '%s'. Try '-m' if audio is at fault.",
-          SDL_GetError());
-  }
-  atexit(SDL_Quit);
-
-  createWindow();
-  if (!screen) {
-    error("Could not initialize screen resolution (message: '%s')", SDL_GetError());
-  }
-
-  if (SDL_GetModState() & KMOD_CAPS) {
-    warning("capslock is on, the mouse will be visible and not grabbed");
+  const GLubyte *gl_version = glGetString(GL_VERSION);
+  if (!gl_version) {
+    error("OpenGL context does not work (cannot even obtain GL_VERSION string)");
   }
 
   /* initialize OpenGL setup before we draw anything */
@@ -421,28 +309,13 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
   initGuileInterface();
   generalInit();
 
-  if (audio != 0) soundInit();
+  if (has_audio) soundInit();
   Settings::settings->loadLevelSets();
 
   /* Initialize and activate the correct gameModes */
-  if (touchMode) {
-    char mapname[512];
-
-    snprintf(mapname, sizeof(mapname) - 1, "%s/.trackballs/levels/%s.map", getenv("HOME"),
-             touchName);
-    if (!fileExists(mapname))
-      snprintf(mapname, sizeof(mapname), "%s/levels/%s.map", effectiveShareDir, touchName);
-    if (!fileExists(mapname)) snprintf(mapname, sizeof(mapname), "%s", touchName);
-    printf("Touching map %s\n", mapname);
-    Map *map = new Map(mapname);
-    map->save(mapname, (int)map->startPosition[0], (int)map->startPosition[1]);
-    exit(0);
-  } else if (editMode) {
-    Font::init();
+  if (startInEditMode) {
     GameMode::activate(EditMode::init());
   } else {
-    Font::init();
-
     /* Activate initial mode */
     if (Settings::settings->doSpecialLevel) {
       GameMode::activate(SetupMode::init());
@@ -476,53 +349,57 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
   srand(seed);
   int keyUpReceived = 1;
 
-  bool is_running = true;
-  while (is_running) {
-    theFrameNumber++;
+  double loop_time = 0.;
+  Uint32 zero_sdl_tick = SDL_GetTicks();
+  while (GameMode::current) {
+    double td = PHYSICS_RESOLUTION;
 
-    double td = getTimeDifference(lastDisplayStartTime, displayStartTime);
-    // If timing doesn't work assume 60fps
-    if (td <= 0.0) { td = 1. / 60.; }
-    // No slower than 5 fps
-    if (td > 0.2) td = 0.2;
+    double real_sdl_time = (SDL_GetTicks() - zero_sdl_tick) * 0.001;
 
-    /* update font system */
-    Font::tick(td);
+    // When game logic has caught up to real time, display
+    if (loop_time > real_sdl_time) {
+      displayFrameNumber++;
 
-    /* Update world */
-    if (GameMode::current) GameMode::current->idle(td);
+      /* Make sure music is still playing */
+      soundIdle(td);
 
-    /* Make sure music is still playing */
-    soundIdle(td);
+      /* update font system */
+      update2DStringCache(false);
 
-    /* Draw world */
-    glViewport(0, 0, screenWidth, screenHeight);
-    if (GameMode::current) {
-      GameMode::current->display();
-    } else {
-      glClear(GL_COLOR_BUFFER_BIT);
+      /* Draw world */
+      glViewport(0, 0, screenWidth, screenHeight);
+      if (GameMode::current) {
+        GameMode::current->display();
+      } else {
+        glClear(GL_COLOR_BUFFER_BIT);
+      }
+      warnForGLerrors("uncaught from display routine");
+      SDL_GL_SetSwapInterval(Settings::settings->vsynced ? 1 : 0);
+      SDL_GL_SwapWindow(window);
+
+      lastDisplayStartTime = displayStartTime;
+      displayStartTime = getMonotonicTime();
+      /* Expensive computations has to be done *after* tick+draw to keep world in good
+         synchronisation. */
+      if (GameMode::current) GameMode::current->doExpensiveComputations();
+      warnForGLerrors("uncaught from expensive computation");
     }
-    warnForGLerrors("uncaught from display routine");
-    SDL_GL_SwapWindow(window);
 
-    lastDisplayStartTime = displayStartTime;
-    displayStartTime = getMonotonicTime();
-    /* Expensive computations has to be done *after* tick+draw to keep world in good
-       synchronisation. */
-    if (GameMode::current) GameMode::current->doExpensiveComputations();
-    warnForGLerrors("uncaught from expensive computation");
+    /* Perform game logic updates, and respond to events */
+    if (GameMode::current) GameMode::current->tick(td);
+    Uint32 until_tick = (Uint32)(loop_time * 1e3) + zero_sdl_tick;
 
-    /*                */
-    /* Process events */
-    /*                */
-    SDL_MouseButtonEvent *e = (SDL_MouseButtonEvent *)&event;
+    /* Respond to events between this logic update and the next */
+    // TODO: react to window events *immediately* -- use two queues?
+    SDL_Event event;
     while (SDL_PollEvent(&event)) {
       switch (event.type) {
       case SDL_QUIT:
-        is_running = false;
+        GameMode::activate(NULL);
         break;
       case SDL_MOUSEBUTTONDOWN:
-        if (GameMode::current) GameMode::current->mouseDown(e->button, e->x, e->y);
+        if (GameMode::current)
+          GameMode::current->mouseDown(event.button.button, event.button.x, event.button.y);
         break;
       case SDL_KEYUP:
         /* Prevent repeated keys */
@@ -535,21 +412,22 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
           } else {
             SDL_ShowCursor(SDL_DISABLE);
           }
-        } else
+        } else if (GameMode::current) {
           GameMode::current->keyUp(event.key.keysym.sym);
+        }
         break;
       case SDL_KEYDOWN:
 
         /* Always quit if the 'q' key is pressed */
-        if (event.key.keysym.sym == 'q' && SDL_GetModState() & KMOD_CTRL) exit(0);
+        if (event.key.keysym.sym == 'q' && SDL_GetModState() & KMOD_CTRL) {
+          GameMode::activate(NULL);
+        }
 
         /* Change between fullscreen/windowed mode if the 'f' key
            is pressed */
         else if (event.key.keysym.sym == 'f' && SDL_GetModState() & KMOD_CTRL) {
           Settings::settings->is_windowed = Settings::settings->is_windowed ? 0 : 1;
           changeScreenResolution();
-          /* Flush all events that occured while switching screen resolution */
-          while (SDL_PollEvent(&event)) {}
         }
 
         /* Use CapsLock key to determine if mouse should be hidden+grabbed */
@@ -562,15 +440,12 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
         }
 
         else if (event.key.keysym.sym == SDLK_ESCAPE) {
-          if (editMode) {
-            ((EditMode *)GameMode::current)->askQuit();
+          if (EditMode::editMode && GameMode::current == EditMode::editMode) {
+            EditMode::editMode->askQuit();
           } else if ((GameMode::current && GameMode::current == MenuMode::menuMode))
-            is_running = false;
+            GameMode::activate(NULL);
           else {
             GameMode::activate(MenuMode::init());
-            /* Flush all events that occured while switching screen
-             resolution */
-            while (SDL_PollEvent(&event)) {}
           }
 
         } else if (GameMode::current) {
@@ -583,17 +458,29 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
 
         break;
       case SDL_WINDOWEVENT:
-        if (event.window.event == SDL_WINDOWEVENT_RESIZED) {
-          /* Only change screen resolution if resizing is enabled */
-          if (Settings::settings->resolution < 0) {
-            changeScreenResolution();
-            /* Flush all events that occured while switching screen resolution */
-            while (SDL_PollEvent(&event)) {}
+        if (event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED) {
+          if (Settings::settings->resolution >= 0) {
+            // Assume fixed resolution unconditionally
+            screenWidth = screenResolutions[Settings::settings->resolution][0];
+            screenHeight = screenResolutions[Settings::settings->resolution][1];
+          } else {
+            // Region drawn resizes with window
+            screenWidth = event.window.data1;
+            screenHeight = event.window.data2;
           }
+
+          /* Adjust for size change in editmode */
+          if (EditMode::editMode) { EditMode::editMode->resizeWindows(); }
         }
         break;
       }
     }
+
+    /* The loop time must reasonably-consistently track real time, and in the
+     * worst case is capped to lag ~100ms behind; otherwise, the loop time
+     * is updated in concert with the simulation time */
+    loop_time =
+        std::max(real_sdl_time - 0.100, loop_time + PHYSICS_RESOLUTION / timeDilationFactor);
   }
 
   Settings::settings->closeJoystick();
@@ -613,11 +500,30 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
   HighScore::cleanup();
   glHelpCleanup();
 
-  SDL_Quit();
+  SDL_GL_DeleteContext(context);
+
+  return (void *)EXIT_SUCCESS;
+}
+
+static void *initSettings(void *) {
+  Settings::init();
+  return NULL;
 }
 
 int main(int argc, char **argv) {
-  program_name = argv[0];
+  const char *program_name = argv[0];
+
+  /* Store the username, for use as a default player/level author name */
+  const char *env_user = getenv("USER");
+  if (env_user) {
+    snprintf(username, sizeof(username) - 1, "%s", env_user);
+  }
+#ifdef WIN32
+  else {
+    const char *env_username = getenv("USERNAME");
+    if (env_username) { snprintf(name, 20, "%s", env_username); }
+  }
+#endif
 
   /*** Autmatic detection of SHARE_DIR ***/
   effectiveShareDir[0] = 0;
@@ -671,6 +577,14 @@ int main(int argc, char **argv) {
     }
   }
 
+  snprintf(effectiveLocalDir, sizeof(effectiveLocalDir), "%s/.trackballs", getenv("HOME"));
+  if (pathIsLink(effectiveLocalDir)) {
+    warning("Error, %s is a symbolic link. Cannot save settings", effectiveLocalDir);
+    return EXIT_FAILURE;
+  }
+  if (!pathIsDir(effectiveLocalDir))
+    mkdir(effectiveLocalDir, S_IXUSR | S_IRUSR | S_IWUSR | S_IXGRP | S_IRGRP | S_IWGRP);
+
   if (NULL == getenv("GUILE_LOAD_PATH")) {
     char guileLoadPath[256 + 16]; /*longest effective share directory plus"GUILE_LOAD_PATH="*/
     snprintf(guileLoadPath, sizeof(guileLoadPath), "GUILE_LOAD_PATH=%s", effectiveShareDir);
@@ -703,6 +617,152 @@ int main(int argc, char **argv) {
   bindtextdomain(PACKAGE, localedir);
   textdomain(PACKAGE);
 
-  scm_boot_guile(argc, argv, innerMain, 0);
-  return 0;
+  /* Loading the settings uses Guile for parsing, but we don't need it again
+   * until e.g. loading the highscores/gamer info, which is done in the non-input
+   * thread */
+  scm_with_guile(initSettings, NULL);
+
+  startInEditMode = false;
+  bool touchMode = false;
+  int audio = SDL_INIT_AUDIO;
+  SDL_Event event;
+  char *touchName = 0;
+
+  const char *const short_options = "he:l:t:wmr:s:fqvyj";
+  const struct option long_options[] = {{"help", 0, NULL, 'h'},
+                                        {"level", 1, NULL, 'l'},
+                                        {"windowed", 0, NULL, 'w'},
+                                        {"mute", 0, NULL, 'm'},
+                                        {"resolution", 1, NULL, 'r'},
+                                        {"sensitivity", 1, NULL, 's'},
+                                        {"fps", 0, NULL, 'f'},
+                                        {"version", 0, NULL, 'v'},
+                                        {"touch", 1, NULL, 't'},
+                                        {"low-memory", 0, NULL, 'y'},
+                                        {"debug-joystick", 0, NULL, '9'},
+                                        {"repair-joystick", 0, NULL, 'j'},
+                                        {NULL, 0, NULL, 0}};
+  int next_option;
+
+  program_name = argv[0];
+
+  displayStartTime = getMonotonicTime();
+  lastDisplayStartTime = displayStartTime;
+  lastDisplayStartTime.tv_sec -= 1;
+  Settings *settings = Settings::settings;
+  settings->doSpecialLevel = 0;
+  settings->setLocale(); /* Start "correct" i18n as soon as possible */
+  low_memory = 0;
+  debug_joystick = 0;
+  repair_joystick = 0;
+  do {
+#if defined(__SVR4) && defined(__sun)
+    next_option = getopt(argc, argv, short_options);
+#else
+    next_option = getopt_long(argc, argv, short_options, long_options, NULL);
+#endif
+
+    int i;
+    switch (next_option) {
+    case 'h':
+      print_usage(stdout, program_name);
+      return EXIT_SUCCESS;
+    case 'l':
+      snprintf(Settings::settings->specialLevel, sizeof(Settings::settings->specialLevel) - 1,
+               "%s", optarg);
+      Settings::settings->doSpecialLevel = 1;
+      break;
+    case 't':
+      touchMode = true;
+      touchName = optarg;
+      has_audio = false;  // no audio
+      break;
+    case 'w':
+      settings->is_windowed = 1;
+      break;
+    case 'm':
+      has_audio = false;
+      break;
+    case 'r':
+      for (i = 0; i < nScreenResolutions; i++)
+        if (screenResolutions[i][0] == atoi(optarg)) break;
+      if (i < nScreenResolutions)
+        settings->resolution = i;
+      else {
+        char estr[256];
+        snprintf(estr, 255, _("Unknown screen resolution of width %d"), i);
+        printf("%s\n", estr);
+      }
+      break;
+    case 's':
+      Settings::settings->mouseSensitivity = atof(optarg);
+      break;
+    case 'f':
+      Settings::settings->showFPS = 1;
+      break;
+    case '?':
+      print_usage(stderr, program_name);
+      return EXIT_FAILURE;
+    case -1:
+      break;
+    case 'v':
+      printf("%s v%s\n", PACKAGE, VERSION);
+      return EXIT_SUCCESS;
+    case 'y':
+      low_memory = 1;
+      break;
+    case '9':
+      debug_joystick = 1;
+      break;
+    case 'j':
+      repair_joystick = 1;
+      break;
+    default:
+      print_usage(stderr, program_name);
+      return EXIT_FAILURE;
+    }
+  } while (next_option != -1);
+
+  printf("%s\n", _("Welcome to Trackballs."));
+  char str[256];
+  snprintf(str, 255, _("Using %s as gamedata directory."), effectiveShareDir);
+  printf("%s\n", str);
+
+  if (touchMode) {
+    /* We do not need any of SDL to load and save a map */
+    char mapname[512];
+
+    snprintf(mapname, sizeof(mapname) - 1, "%s/levels/%s.map", effectiveLocalDir, touchName);
+    if (!fileExists(mapname))
+      snprintf(mapname, sizeof(mapname), "%s/levels/%s.map", effectiveShareDir, touchName);
+    if (!fileExists(mapname)) snprintf(mapname, sizeof(mapname), "%s", touchName);
+    printf("Touching map %s\n", mapname);
+    Map *map = new Map(mapname);
+    map->save(mapname, (int)map->startPosition[0], (int)map->startPosition[1]);
+    delete map;
+    return EXIT_SUCCESS;
+  }
+
+  /* Initialize SDL */
+  if ((SDL_Init(SDL_INIT_VIDEO | (has_audio ? SDL_INIT_AUDIO : 0) | SDL_INIT_JOYSTICK) ==
+       -1)) {
+    error("Could not initialize libSDL. Error message: '%s'. Try '-m' if audio is at fault.",
+          SDL_GetError());
+  }
+  atexit(SDL_Quit);
+
+  SDL_GLContext gl_context = createWindow();
+  if (!gl_context) {
+    error("Could not initialize screen resolution (message: '%s')", SDL_GetError());
+  }
+
+  if (SDL_GetModState() & KMOD_CAPS) {
+    warning("capslock is on, the mouse will be visible and not grabbed");
+  }
+
+  void *v = scm_with_guile(mainLoop, gl_context);
+  int retval = (int)(ptrdiff_t)v;
+
+  SDL_Quit();
+  return retval;
 }
