@@ -28,7 +28,7 @@
 
 class Pipe : public Animated {
  public:
-  Pipe(const Coord3d& from, const Coord3d& to, Real radius);
+  Pipe(Game& g, const Coord3d& from, const Coord3d& to, Real radius);
 
   virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
   virtual void drawBuffers1(const GLuint*) const;
