@@ -25,6 +25,7 @@
 #include "game.h"
 #include "mainMode.h"
 #include "map.h"
+#include "replay.h"
 #include "settings.h"
 #include "sound.h"
 #include "splash.h"
@@ -32,7 +33,7 @@
 #include <SDL2/SDL_keyboard.h>
 #include <SDL2/SDL_mouse.h>
 
-static Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
+static Uint32 getFilteredRelativeMouse(double now, double *xrate, double *yrate) {
   /* Finite impulse response filter to smooth mouse input */
   const int N = 1000;
   const double T = 0.2;
@@ -48,8 +49,6 @@ static Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
     first = false;
   }
 
-  double now = Game::current->gameTime;
-
   int mouseX, mouseY;
   Uint32 mouseState = SDL_GetRelativeMouseState(&mouseX, &mouseY);
   if (mouseX || mouseY) {
@@ -83,7 +82,7 @@ static Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
   return mouseState;
 }
 
-Player::Player() : Ball(Role_Player) {
+Player::Player(Game &g) : Ball(g, Role_Player) {
   inTheAir = false;
   inPipe = false;
   lives = 3;
@@ -106,7 +105,12 @@ Player::Player() : Ball(Role_Player) {
   oxygen = 1.0;
   moveBurst = 0.0;
 
-  lastJoyX = lastJoyY = 0;
+  control.dx = 0.;
+  control.dy = 0.;
+  control.jump = false;
+  control.inactive = true;
+  control.die = false;
+
   setReflectivity(0.4, 0);
 
   scoreOnDeath = Game::defaultScores[SCORE_PLAYER][0];
@@ -120,10 +124,9 @@ void Player::tick(Real t) {
   /* Never let us drop below 0 points, it just looks silly */
   if (score < 0) score = 0;
 
-  if (!Game::current) return;
   if (!playing) return;
 
-  Map *map = Game::current->map;
+  Map *map = game.map;
   health += t * 0.4;
   if (health > 1.0) health = 1.0;
 
@@ -157,85 +160,34 @@ void Player::tick(Real t) {
   */
   if (map->getWaterHeight(position[0], position[1]) > position[2] + radius * 0.75 &&
       (!(inPipe && position[2] < map->getHeight(position[0], position[1])))) {
-    oxygen -=
-        (t / (12.0 - 2.0 * Settings::settings->difficulty)) / Game::current->oxygenFactor;
+    oxygen -= (t / (12.0 - 2.0 * Settings::settings->difficulty)) / game.oxygenFactor;
     if (oxygen <= 0.0) die(DIE_OTHER);
   } else
-    oxygen = std::min(1.0, oxygen + (t / 4.0) / Game::current->oxygenFactor);
+    oxygen = std::min(1.0, oxygen + (t / 4.0) / game.oxygenFactor);
 
-  double dx = 0., dy = 0.;
-  /* Joysticks */
-  if (Settings::settings->hasJoystick()) {
-    if (Settings::settings->joystickButton(0)) key(' ');
-
-    double joyX = Settings::settings->joystickX();
-    double joyY = Settings::settings->joystickY();
-    /* Help keep the joysticks centered if neccessary */
-    if (joyX < 0.1 && joyX > -0.1) joyX = 0.0;
-    if (joyY < 0.1 && joyY > -0.1) joyY = 0.0;
-
-    dx = ((double)joyX) * Settings::settings->mouseSensitivity;
-    dy = ((double)joyY) * Settings::settings->mouseSensitivity;
-  }
-
-  /* Give only *relative* mouse movements */
-  if (!Settings::settings->ignoreMouse && !(SDL_GetModState() & KMOD_CAPS)) {
-    double sx, sy;
-    getFilteredRelativeMouse(&sx, &sy);
-    if (sx || sy) {
-      dx = sx * Settings::settings->mouseSensitivity * 0.0005;
-      dy = sy * Settings::settings->mouseSensitivity * 0.0005;
-    }
-  }
-
-  /* Handle keyboard steering */
-  const Uint8 *keystate = SDL_GetKeyboardState(NULL);
-  int shift = SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT);
-
-  int ix = 0, iy = 0;
-  if (keystate[SDL_SCANCODE_UP] || keystate[SDL_SCANCODE_KP_8]) iy--;
-  if (keystate[SDL_SCANCODE_DOWN] || keystate[SDL_SCANCODE_KP_2]) iy++;
-  if (keystate[SDL_SCANCODE_LEFT] || keystate[SDL_SCANCODE_KP_4]) ix--;
-  if (keystate[SDL_SCANCODE_RIGHT] || keystate[SDL_SCANCODE_KP_6]) ix++;
-  if (keystate[SDL_SCANCODE_KP_7] || keystate[SDL_SCANCODE_Q]) {
-    ix--;
-    iy--;
-  }
-  if (keystate[SDL_SCANCODE_KP_9] || keystate[SDL_SCANCODE_W]) {
-    ix++;
-    iy--;
-  }
-  if (keystate[SDL_SCANCODE_KP_1] || keystate[SDL_SCANCODE_A]) {
-    ix--;
-    iy++;
-  }
-  if (keystate[SDL_SCANCODE_KP_3] || keystate[SDL_SCANCODE_S]) {
-    ix++;
-    iy++;
-  }
-  if (ix || iy) {
-    double len = std::sqrt(ix * ix + iy * iy);
-    dx = ix / len;
-    dy = iy / len;
+  if (Settings::settings->replay) { control = game.replayStore.get(game.gameTicks); }
+  double dx = control.dx, dy = control.dy;
+  if (Settings::settings->storeReplay && !Settings::settings->replay) {
+    game.replayStore.add(control);
   }
 
   /* rotate control as by settings->rotateArrows */
   {
     double angle = Settings::settings->rotateSteering * M_PI / 4.0;
-    double tmp = dx * cos(angle) - dy * sin(angle);
-    dy = dy * cos(angle) + dx * sin(angle);
+    double tmp = dx * std::cos(angle) - dy * std::sin(angle);
+    dy = dy * std::cos(angle) + dx * std::sin(angle);
     dx = tmp;
   }
 
   /* rotate controls if the camera perspective is rotated */
-  double angle = ((MainMode *)GameMode::current)->xyAngle * M_PI / 2.;
-  if (angle) {
-    double tmp = dx * cos(angle) - dy * sin(angle);
-    dy = dy * cos(angle) + dx * sin(angle);
+  if (GameMode::current == MainMode::mainMode) {
+    double angle = MainMode::mainMode->xyAngle * M_PI / 2.;
+    double tmp = dx * std::cos(angle) - dy * std::sin(angle);
+    dy = dy * std::cos(angle) + dx * std::sin(angle);
     dx = tmp;
   }
 
-  /* Cap dx/dy to have total radius 1. */
+  /* Cap dx/dy to have total radius 1, in case input yields extreme input */
   double len = std::sqrt(dx * dx + dy * dy);
   if (len > 1.) {
     dx /= len;
@@ -248,11 +200,6 @@ void Player::tick(Real t) {
   dx = tx;
   dy = ty;
 
-  if (shift) {
-    dx *= 0.5;
-    dy *= 0.5;
-  }
-
   /* set inherent base acceleration*/
   acceleration = 3.5 - 0.25 * Settings::settings->difficulty;
 
@@ -265,27 +212,23 @@ void Player::tick(Real t) {
     friction = 1.0;
   else
     friction = 10.0;
+
+  if (control.jump) {
+    double jumpStrength = game.jumpFactor * (1.2 - 0.1 * Settings::settings->difficulty);
+    jumpStrength *= modTimeLeft[MOD_JUMP] ? 5.0 : 3.0;
+    Ball::queueJump(jumpStrength);
+    control.jump = false;
+  }
   Ball::tick(t);
-}
-void Player::key(int c) {
-  switch (c) {
-  case ' ':
-    jump();
-    break;
-  case 'k':
+
+  if (control.die) {
     die(DIE_OTHER);
-    break;
+    control.die = false;
   }
-}
-void Player::jump() {
-  double jumpStrength =
-      Game::current->jumpFactor * (1.2 - 0.1 * Settings::settings->difficulty);
-  jumpStrength *= modTimeLeft[MOD_JUMP] ? 5.0 : 3.0;
-  Ball::queueJump(jumpStrength);
 }
 
 void Player::die(int how) {
-  Map *map = Game::current->map;
+  Map *map = game.map;
   /* immortal when not playing (i.e., at level finish) */
   if (hasWon) return;
   if (!playing) return;
@@ -294,7 +237,7 @@ void Player::die(int how) {
   /* death hooks may have altered `hasWon` and `playing` */
   if (hasWon) return;
   if (!playing) return;
-  if (Game::current->map->isBonus) {
+  if (game.map->isBonus) {
     MainMode::mainMode->bonusLevelComplete();
     return;
   }
@@ -312,20 +255,20 @@ void Player::die(int how) {
     Color acidColor(0.1, 0.8, 0.1, 0.5);
     Coord3d vel(0., 0., 1.);
     Coord3d center(position[0], position[1], map->getHeight(position[0], position[1]));
-    Game::current->add(new Splash(center, vel, acidColor, 32.0, radius));
+    game.add(new Splash(game, center, vel, acidColor, 32.0, radius));
   } else {
     for (int i = 0; i < 4; i++) {
       for (int j = 0; j < 4; j++) {
         Real a = i / 4.0 * M_PI2;
         Real b = (j + 0.5) / 4.0 * M_PI;
         Coord3d pos, vel;
-        pos[0] = position[0] + cos(a) * 0.25 * sin(b) * 2.0;
-        pos[1] = position[1] + sin(a) * 0.25 * sin(b) * 2.0;
-        pos[2] = position[2] + 0.25 * cos(b) + 0.5;
-        vel[0] = velocity[0] + (sink ? 0.1 : 0.5) * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        vel[1] = velocity[1] + (sink ? 0.1 : 0.5) * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        vel[2] = velocity[2] + (sink ? 0.01 : 0.5) * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        Game::current->add(new Debris(this, pos, vel, 2.0 + 8.0 * frandom()));
+        pos[0] = position[0] + std::cos(a) * 0.25 * std::sin(b) * 2.0;
+        pos[1] = position[1] + std::sin(a) * 0.25 * std::sin(b) * 2.0;
+        pos[2] = position[2] + 0.25 * std::cos(b) + 0.5;
+        vel[0] = velocity[0] + (sink ? 0.2 : 1.0) * (game.frandom() - 0.5);
+        vel[1] = velocity[1] + (sink ? 0.2 : 1.0) * (game.frandom() - 0.5);
+        vel[2] = velocity[2] + (sink ? 0.02 : 1.0) * (game.frandom() - 0.5);
+        game.add(new Debris(game, this, pos, vel, 2.0 + 8.0 * game.frandom()));
       }
     }
   }
@@ -361,13 +304,108 @@ void Player::setStartVariables() {
 void Player::restart(const Coord3d &pos) {
   setStartVariables();
   position = pos;
-  position[2] = Game::current->map->getHeight(position[0], position[1]) + radius;
+  position[2] = game.map->getHeight(position[0], position[1]) + radius;
   modTimeLeft[MOD_DIZZY] = 0.0;
 
   /* reset all mods */
   /*for(i=0;i<NUM_MODS;i++)
     modTimeLeft[i] = 0.0;*/
 }
+void Player::handleKey(int key) {
+  if (!Settings::settings->replay) {
+    if (key == 'k') { control.die = true; }
+    if (key == ' ') { control.jump = true; }
+  }
+}
+void Player::handleUserInput(bool active) {
+  control.inactive = !active;
+  /* Priority order for input: Joystick >> Keyboard >> Mouse
+   *
+   * If any input category provides zero input, we fall back to
+   * the next category.
+   */
+  /* Mouse filtering is continuous */
+  double sx = 0., sy = 0.;
+  if (!Settings::settings->ignoreMouse && !(SDL_GetModState() & KMOD_CAPS)) {
+    getFilteredRelativeMouse(game.gameTime, &sx, &sy);
+  }
+  /* Do *not* handle input when not playing */
+  if (!playing) { return; }
+
+  /* Joysticks */
+  if (Settings::settings->hasJoystick()) {
+    // warning: jump should be edge-triggered
+    if (Settings::settings->joystickButton(0)) control.jump = true;
+
+    double joyX = Settings::settings->joystickX();
+    double joyY = Settings::settings->joystickY();
+    /* Help keep the joysticks centered if neccessary */
+    if (joyX < 0.1 && joyX > -0.1) joyX = 0.0;
+    if (joyY < 0.1 && joyY > -0.1) joyY = 0.0;
+
+    if (joyX != 0.0 && joyY != 0.0) {
+      control.dx = ((double)joyX) * Settings::settings->mouseSensitivity;
+      control.dy = ((double)joyY) * Settings::settings->mouseSensitivity;
+      return;
+    }
+  }
+
+  /* Handle keyboard steering */
+  const Uint8 *keystate = SDL_GetKeyboardState(NULL);
+  bool precise = SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT);
+
+  int ix = 0, iy = 0;
+  if (keystate[SDL_SCANCODE_UP] || keystate[SDL_SCANCODE_KP_8]) iy--;
+  if (keystate[SDL_SCANCODE_DOWN] || keystate[SDL_SCANCODE_KP_2]) iy++;
+  if (keystate[SDL_SCANCODE_LEFT] || keystate[SDL_SCANCODE_KP_4]) ix--;
+  if (keystate[SDL_SCANCODE_RIGHT] || keystate[SDL_SCANCODE_KP_6]) ix++;
+  if (keystate[SDL_SCANCODE_KP_7] || keystate[SDL_SCANCODE_Q]) {
+    ix--;
+    iy--;
+  }
+  if (keystate[SDL_SCANCODE_KP_9] || keystate[SDL_SCANCODE_W]) {
+    ix++;
+    iy--;
+  }
+  if (keystate[SDL_SCANCODE_KP_1] || keystate[SDL_SCANCODE_A]) {
+    ix--;
+    iy++;
+  }
+  if (keystate[SDL_SCANCODE_KP_3] || keystate[SDL_SCANCODE_S]) {
+    ix++;
+    iy++;
+  }
+  if (ix || iy) {
+    double len = std::sqrt(ix * ix + iy * iy);
+    control.dx = ix / len;
+    control.dy = iy / len;
+    if (precise) {
+      control.dx /= 2;
+      control.dy /= 2;
+    }
+    return;
+  }
+
+  /* Give only *relative* mouse movements */
+  if (!Settings::settings->ignoreMouse && !(SDL_GetModState() & KMOD_CAPS)) {
+    if (sx != 0. || sy != 0.) {
+      control.dx = sx * Settings::settings->mouseSensitivity * 0.0005;
+      control.dy = sy * Settings::settings->mouseSensitivity * 0.0005;
+      double len = std::max(1., std::sqrt(control.dx * control.dx + control.dy * control.dy));
+      control.dx /= len;
+      control.dy /= len;
+      if (precise) {
+        control.dx /= 2;
+        control.dy /= 2;
+      }
+      return;
+    }
+  }
+
+  /* No input from any source, do nothing */
+  control.dx = 0.;
+  control.dy = 0.;
+}
 void Player::mouse(int /*state*/, int /*x*/, int /*y*/) {}
 void Player::newLevel() {}
 void Player::setHealth(Real d) {
