@@ -27,10 +27,11 @@
 #include <SDL2/SDL_joystick.h>
 #include <dirent.h>
 #include <libguile.h>
-#include <locale.h>
-#include <string.h>
 #include <sys/stat.h>
+#include <unistd.h>
+#include <clocale>
 #include <cstdlib>
+#include <cstring>
 
 extern double timeDilationFactor;
 
@@ -83,12 +84,14 @@ Settings::Settings() {
   shadowTexsize = 9;
   timeCompression = 0;
   sandboxAvailable = 0;
+  storeReplay = 0;
+
   memset(specialLevel, 0, sizeof(specialLevel));
   memset(levelSets, 0, sizeof(levelSets));
 
   /* Load all settings from a scheme-syntaxed config file */
   char str[256];
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs/settings", getenv("HOME"));
+  snprintf(str, sizeof(str) - 1, "%s/settings", effectiveLocalDir);
   if (access(str, R_OK) != -1) {
     SCM ip = scm_open_file(scm_from_utf8_string(str), scm_from_utf8_string("r"));
     // ^ TODO catch exception
@@ -106,21 +109,21 @@ Settings::Settings() {
         warning("Configuration file should be a series of (key value) tuples.");
         continue;
       }
-      const int intnum = 21, realnum = 3;
+      const int intnum = 22, realnum = 3;
       const char *intkeys[intnum] = {
-          "gfx-details",      "show-fps",       "is-windowed",    "resolution",
-          "color-depth",      "joystick-index", "joy_center-x",   "joy_center-y",
-          "joy-left",         "joy-right",      "joy-up",         "joy-down",
-          "rotate-steering",  "language",       "ignore-mouse",   "do-reflections",
-          "do-shadows",       "vsync-on",       "shadow-texsize", "time-compression",
-          "sandbox-available"};
+          "gfx-details",       "show-fps",       "is-windowed",    "resolution",
+          "color-depth",       "joystick-index", "joy_center-x",   "joy_center-y",
+          "joy-left",          "joy-right",      "joy-up",         "joy-down",
+          "rotate-steering",   "language",       "ignore-mouse",   "do-reflections",
+          "do-shadows",        "vsync-on",       "shadow-texsize", "time-compression",
+          "sandbox-available", "store-replay"};
 
       int *intdests[intnum] = {
-          &gfx_details,     &showFPS,       &is_windowed,    &resolution,    &colorDepth,
-          &joystickIndex,   &joy_center[0], &joy_center[1],  &joy_left,      &joy_right,
-          &joy_up,          &joy_down,      &rotateSteering, &language,      &ignoreMouse,
-          &doReflections,   &doShadows,     &vsynced,        &shadowTexsize, &timeCompression,
-          &sandboxAvailable};
+          &gfx_details,      &showFPS,       &is_windowed,    &resolution,    &colorDepth,
+          &joystickIndex,    &joy_center[0], &joy_center[1],  &joy_left,      &joy_right,
+          &joy_up,           &joy_down,      &rotateSteering, &language,      &ignoreMouse,
+          &doReflections,    &doShadows,     &vsynced,        &shadowTexsize, &timeCompression,
+          &sandboxAvailable, &storeReplay};
       const char *realkeys[realnum] = {"sfx-volume", "music-volume", "mouse-sensitivity"};
       double *realdests[realnum] = {&sfxVolume, &musicVolume, &mouseSensitivity};
 
@@ -183,15 +186,14 @@ void Settings::loadLevelSets() {
     closedir(dir);
   }
 
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs/levels", getenv("HOME"));
+  snprintf(str, sizeof(str) - 1, "%s/levels", effectiveLocalDir);
   dir = opendir(str);
   if (dir) {
     struct dirent *dirent;
     while ((dirent = readdir(dir))) {
       if (strlen(dirent->d_name) > 4 &&
           strcmp(&dirent->d_name[strlen(dirent->d_name) - 4], ".set") == 0) {
-        snprintf(str, sizeof(str) - 1, "%s/.trackballs/levels/%s", getenv("HOME"),
-                 dirent->d_name);
+        snprintf(str, sizeof(str) - 1, "%s/levels/%s", effectiveLocalDir, dirent->d_name);
         loadLevelSet(str, dirent->d_name);
       }
     }
@@ -302,15 +304,7 @@ void Settings::loadLevelSet(const char *setname, const char *shortname) {
 
 void Settings::save() {
   char str[256];
-
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs", getenv("HOME"));
-  if (pathIsLink(str)) {
-    warning("%s is a symbolic link. Cannot save settings", str);
-    return;
-  }
-
-  mkdir(str, S_IXUSR | S_IRUSR | S_IWUSR | S_IXGRP | S_IRGRP | S_IWGRP);
-  snprintf(str, sizeof(str) - 1, "%s/.trackballs/settings", getenv("HOME"));
+  snprintf(str, sizeof(str) - 1, "%s/settings", effectiveLocalDir);
   if (pathIsLink(str)) {
     warning("%s is a symbolic link. Cannot save settings", str);
     return;
@@ -345,6 +339,7 @@ void Settings::save() {
     fprintf(fp, "(vsync-on %d)\n", vsynced);
     fprintf(fp, "(time-compression %d)\n", timeCompression);
     fprintf(fp, "(sandbox-available %d)\n", sandboxAvailable);
+    fprintf(fp, "(store-replay %d)\n", storeReplay);
     fclose(fp);
   }
 }
