@@ -30,7 +30,7 @@ class SettingsMode : public GameMode {
   void activated();
   void deactivated();
   void key(int);
-  void idle(Real);
+  void tick(Real);
   void mouseDown(int state, int x, int y);
 
   static SettingsMode *init();
@@ -70,6 +70,7 @@ class SettingsMode : public GameMode {
     MENU_LANGUAGE,
     MENU_SANDBOX_AVAILABLE,
     MENU_TIME_COMPRESSION,
+    MENU_STORE_REPLAY,
 
     MENU_RESOLUTION_OK,
     MENU_RESOLUTION_BAD,
