@@ -25,6 +25,7 @@
 
 #include <SDL2/SDL_mixer.h>
 #include <dirent.h>
+#include <algorithm>  // for std::min/max
 
 SDL_AudioSpec audioFormat;
 
@@ -108,14 +109,14 @@ void soundIdle(Real td) {
   }
 
   if (doSpecialSfx) {
-    musicFade = fmax(0.0, musicFade - 0.3 * td);
+    musicFade = std::max(0.0, musicFade - 0.3 * td);
     if (musicFade == 0.0) {
       Mix_Volume(1, (int)(128 * Settings::settings->sfxVolume));
       Mix_PlayChannel(1, effects[doSpecialSfx], 0);
       doSpecialSfx = 0;
     }
   } else if (!Mix_Playing(1))
-    musicFade = fmin(1.0, musicFade + 0.3 * td);
+    musicFade = std::min(1.0, musicFade + 0.3 * td);
 
   if (!nomusic) {
     Mix_VolumeMusic((int)(musicFade * 127.0 * Settings::settings->musicVolume));
