@@ -28,10 +28,10 @@
 #include "player.h"
 #include "sound.h"
 
-Spike::Spike(const Coord3d &position, Real speed, Real phase)
-    : Animated(Role_OtherAnimated, 1) {
-  this->position = position;
-  this->position[2] = Game::current->map->getHeight(position[0], position[1]) + 0.0;
+Spike::Spike(Game &g, const Coord3d &pos, Real speed, Real phase)
+    : Animated(g, Role_OtherAnimated, 1) {
+  this->position = pos;
+  this->position[2] = game.map->getHeight(pos[0], pos[1]) + 0.0;
   this->speed = speed;
   while (phase < 0.0) phase += 1.0;
   this->phase = phase;
@@ -79,10 +79,10 @@ void Spike::tick(Real t) {
 
   int is_sinking = 0;
 
-  double x = position[0], y = position[1], z = position[2];
   if (is_on) phase += t * this->speed;
   while (phase > 1.0) phase -= 1.0;
-  double h = Game::current->map->getHeight(x, y) - 0.3;
+  double h = game.map->getHeight(position[0], position[1]) - 0.3;
+  double z;
   if (phase < 0.4) {
     /* spike is low */
     z = h;  //
@@ -101,7 +101,8 @@ void Spike::tick(Real t) {
   position[2] = z;
 
   Animated **balls;
-  int nballs = Game::current->balls->bboxOverlapsWith(this, &balls);
+  double x = position[0], y = position[1];
+  int nballs = game.balls->bboxOverlapsWith(this, &balls);
   for (int i = 0; i < nballs; i++) {
     Ball *ball = (Ball *)balls[i];
 
@@ -109,7 +110,7 @@ void Spike::tick(Real t) {
         ball->position[1] < y + 1) {
       double dx = ball->position[0] - x;
       double dy = ball->position[1] - y;
-      double dist = sqrt(dx * dx + dy * dy);
+      double dist = std::sqrt(dx * dx + dy * dy);
 
       if (dist < 0.1 + ball->radius) {
         h = ball->position[2] + dist;
@@ -130,10 +131,10 @@ void Spike::tick(Real t) {
   }
 
   // play a 'rising' sound if the ball is in the round
-  Player *player = Game::current->player1;
+  Player *player = game.player1;
   double dist =
-      sqrt((position[0] - player->position[0]) * (position[0] - player->position[0]) +
-           (position[1] - player->position[1]) * (position[1] - player->position[1]));
+      std::sqrt((position[0] - player->position[0]) * (position[0] - player->position[0]) +
+                (position[1] - player->position[1]) * (position[1] - player->position[1]));
   if ((dist < 9.) && (!soundDone) && (phase >= 0.4) && (phase < 0.5)) {
     if (dist < 6.)
       playEffect(SFX_SPIKE, 0.66);
@@ -148,7 +149,7 @@ void generateSpikeVBO(GLfloat *data, ushort idxs[][3], Matrix3d rotmtx,
   const int nfacets = 6;
   char *pos = (char *)data;
 
-  double d1 = 1 / sqrt(10.0), d2 = 3 / sqrt(10.0);
+  double d1 = 1 / std::sqrt(10.0), d2 = 3 / std::sqrt(10.0);
   for (int i = 0; i < 4 * nfacets; i++) {
     Coord3d local;
     Coord3d normal;
