@@ -26,7 +26,7 @@
 
 class Teleport : public Animated {
  public:
-  Teleport(Real x, Real y, Real dx, Real dy, Real radius);
+  Teleport(Game& g, Real x, Real y, Real dx, Real dy, Real radius);
 
   virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
   virtual void drawBuffers1(const GLuint*) const;
